﻿/**
FormulaOperator.Operator.DivideNode

Copyright (c) 2015 Shigeyuki Horimoto

This software is released under the MIT License.
http://opensource.org/licenses/mit-license.php
*/
using Formula.Node;
using Formula.Node.Impl;
using FormulaOperator.Variable;
using System;

namespace FormulaOperator.Operator
{
    public class DivideNode : AbstractOperatorNode
    {
        private static string[] signature = { "/" };
        public override string[] Signature { get { return signature; } }
        public override int priority { get { return 1; } }
        public override Type[] ArgCheckList { get { return new Type[] { typeof(NumberNode), typeof(NumberNode) }; } }
        public override Type ReturnNode { get { return typeof(NumberNode); } }
        public override bool IsPermitLeftNodeBlank { get { return false; } }
        protected override INode execute()
        {
            decimal? ret = null;

            INode l = this.getLeftNode();
            INode r = this.getRightNode();

            IValueNode lret = l == null ? null : (IValueNode)l.eval();
            IValueNode rret = r == null ? null : (IValueNode)r.eval();

            if (lret != null && lret is ErrorNode) return lret;
            if (rret != null && rret is ErrorNode) return rret;

            decimal? lvalue = lret == null ? null : lret.getDecimal();
            decimal? rvalue = rret == null ? null : rret.getDecimal();

            if (lvalue == null && rvalue == null) ret = null;
            else if (lvalue == null) ret = null;
            else if (rvalue == null) ret = null;
            else ret = lvalue / rvalue;

            NumberNode v = new NumberNode();
            v.Value = ret;
            //v.startPosition = f == null ? -1 : ((AbstractValueNode)f).startPosition;
            //v.endPosition = l == null ? -1 : ((AbstractValueNode)l).endPosition;
            return v;
        }

    }
}
